import os
from flask import Flask, render_template, send_file, redirect, url_for, request, url_for, Blueprint
from flask_bootstrap import Bootstrap
from flask import *
from flask_mysqldb import MySQL
import MySQLdb
from werkzeug.security import generate_password_hash, check_password_hash
from werkzeug.utils import secure_filename
from flask_login import LoginManager, UserMixin, login_user, login_required, logout_user, current_user
from flask_sqlalchemy import SQLAlchemy
from sqlalchemy.exc import IntegrityError
from PIL import Image,UnidentifiedImageError
from io import BytesIO
import io
from flask_mail import Mail, Message
from itsdangerous import URLSafeTimedSerializer, SignatureExpired, BadTimeSignature
from flask_wtf import FlaskForm
from wtforms import StringField, PasswordField
from wtforms.validators import InputRequired, Email, Length

from edit_routes import edit_routes
import cred

host=cred.host,
user=cred.user,
password=cred.password,
database=cred.database
conn = 'mysql+pymsql://{0}:{1}@{2}/{3}'.format(cred.user,cred.password,cred.host,cred.database)


app = Flask(__name__)
app.register_blueprint(edit_routes, url_prefix="")
app.secrete_key='THISISHUSHHUSH'
app.config['SECRET_KEY'] ='THISISHUSHHUSH'
app.config["SQLALCHEMY_DATABASE_URI"] = 'mysql://'+cred.user+':'+cred.password+'@'+cred.host+'/'+cred.user
app.config.from_pyfile('config.cfg')
mail = Mail(app)
serializer = URLSafeTimedSerializer(app.config['SECRET_KEY'])
db = SQLAlchemy(app)
Bootstrap(app)
db = SQLAlchemy(app)
login_manager = LoginManager()
login_manager.init_app(app)
login_manager.login_view = 'login'


# class object to create a user and holds its pertinent information.
# objects in the form be stored in a MySQL database using SQLALCHEMY 
class usernames(UserMixin, db.Model):
    username = db.Column(db.String(15), unique=True)
    password = db.Column(db.String(80))
    id = db.Column(db.Integer, primary_key = True)
    email = db.Column(db.String(35), unique=True)

# Register class holds the information submiited through the signup route
class Register(FlaskForm):
    user = StringField('user', validators=[InputRequired(), Length(min=5, max = 10)])
    email = StringField('email', validators=[InputRequired(), Email(message='Not a Valid Email'), Length(max = 35)])
    password = PasswordField('password', validators=[InputRequired(), Length(min=5,max=80)])

# LoginForm class holds the information submiited through the login route
class LoginForm(FlaskForm):
    user = StringField('user', validators=[InputRequired(), Length(min=5, max = 10)])
    password = PasswordField('password', validators=[InputRequired(), Length(min=5,max=80)])


mysql = MySQL(app)
port = edit_routes.port
data = []

# This sets the callback for reloading a user from the session. The function you set should take a user ID 
# (a unicode) and return a user object, or None if the user does not exist.
@login_manager.user_loader
def load_user(id):
    return usernames.query.get(int(id))


def connect():
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database )
    return mydb

# route that handles user login, grabs information for the client
@app.route('/login', methods = ['GET','POST'])
def login():
    form = LoginForm()
    username=form.user.data
    if form.validate_on_submit():
        user = usernames.query.filter_by(username=form.user.data).first()
        if user:
            if check_password_hash(user.password, form.password.data):
                mydb = MySQLdb.connect(
                host=cred.host,
                user=cred.user,
                password=cred.password,
                database=cred.database )
                mycursor = mydb.cursor()
                mycursor.execute("SELECT verified FROM usernames where username = %s",(username,));
                isVerified = mycursor.fetchall()
                mydb.commit()
                mycursor.close()
                mydb.close

                isVerified=isVerified[0][0]
                
                print(isVerified)
                if isVerified:
                    login_user(user)
                    return redirect(url_for('profile'))        
        return '<h1>Invalid Credentials! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/login" /> </h1>'
    return render_template('login.html',form = form)

# errorhandler that catches all 404 errors, returns to user profile if user is currentlly logged in 
# redirects to Login Screen if user is currently not logged in
@app.errorhandler(404)
def pageNotFound(e):
    if current_user.is_authenticated:
        return '<h1>Page Not Found, Redirecting to User Profile! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/profile" /> </h1>'
    else:
        return '<h1>Page Not Found, Redirecting to Login Screen! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/profile" /> </h1>'
        

# route that enables a logout feture of a currently logged in user
@app.route('/logout')
@login_required
def logout():
    logout_user()
    return redirect(url_for('login'))

# route to enable a user to sign up for an account
# requires a username, password(sha256 encrypted), and email
@app.route('/signup', methods=['GET', 'POST'])
def signup():
    form = Register()
    if form.validate_on_submit():
        hashed_passwd = generate_password_hash(form.password.data, method ='sha256')
        new_user = usernames(username=form.user.data, password = hashed_passwd, email =form.email.data)
        db.session.add(new_user)

        try:
            db.session.commit()
            db.session.close()
        except IntegrityError:
            return render_template("UserAlreadyExists.html",port = port)
            
        email =form.email.data
        new_user = usernames(username=form.user.data, password = hashed_passwd, email =form.email.data)
        token = serializer.dumps(email, salt = 'email-verify')
        msg = Message('Confirmation Email', sender = 'seniorprojectflaskemail@gmail.com', recipients=[email])
        verifyLink = url_for('verifyEmail',token = token, _external = True)
        msg.body = """Please verify your account by clicking the link below: \n
                      LINK: {}  """.format(verifyLink)
        mail.send(msg)


        return '<h1>Check Email for Verification Link <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/login" /> </h1>'
    return render_template('signup.html', form=form)

# Route that handles verifiying the email
@app.route('/verify-email/<token>')
def verifyEmail(token):
    try:
        email = serializer.loads(token, salt='email-verify', max_age = 600)
    except SignatureExpired:
        return '<h1>Link Expired! Try Signing up Again! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/signup" /> </h1>'
    except BadTimeSignature:
        return '<h1>Invalid Verification Email! Try Signing up Again! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/signup" /> </h1>'
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database )
    mycursor = mydb.cursor()
    mycursor.execute("UPDATE usernames SET verified = 1 where email = %s",(email,));
    mydb.commit()
    mycursor.close()
    mydb.close()
    return '<h1>Email Verified, Redircting to Login Screen<meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/login" /> </h1>'
    
# Route that deletes an image if given a name via form. Only can delete the image if user owns the image
@app.route('/deleteByName',methods=['POST'])
@login_required
def deleteByName():
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database )
    mycursor = mydb.cursor()
    nameToDelete = request.form['nameToDelete']
    mycursor.execute("DELETE from images where image_name = %s",(nameToDelete,));
    mydb.commit()
    mycursor.close()
    mydb.close()
    return redirect(url_for('index'))

    
# Route that ranmes an image if given a name via form. Only can rename if user owns the image
@app.route('/rename/<int:id>',methods=['POST'])
@login_required
def rename(id):
#     mydb = MySQLdb.connect(
#     host=cred.host,
#     user=cred.user,
#     password=cred.password,
#     database=cred.database
# )
    mydb = connect()
    aUser = current_user.username
    
    test = request.form['nm']
    mycursor = mydb.cursor()
    mycursor.execute("SELECT image_id from images where image_id = %s and user_id = %s",(id,current_user.id));
    checkOwnerShip = mycursor.rowcount
    mycursor.execute("SELECT image_id from images where image_id = %s",(id,));
    doesExist = mycursor.rowcount

    if not doesExist:
        return '<h1>No Such Image! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/'+str(aUser)+'/gallary"/> </h1>'
    
    elif checkOwnerShip:
        mycursor.execute("UPDATE images SET image_name = %s WHERE image_id =%s",(test,id));
        mydb.commit()
   
    else:
        return '<h1>Not Your File to Rename! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/'+str(aUser)+'/gallary"/> </h1>'
    
    return redirect(url_for('index'))

# checks to see if the file uploaded is an allowable extenstion
app.config["ALLOWED_EXTENSIONS"] = {'png', 'jpg', 'jpeg', 'jfif'}
def allowed_file(filename):
    return '.' in filename and \
        filename.rsplit('.', 1)[1].lower() in app.config["ALLOWED_EXTENSIONS"]


# Directs to image upload page
@app.route('/form')
@login_required
def form():
   return render_template("form.html",name=current_user.username)

# handles image uploading  
@app.route('/upload', methods=['GET','POST'])
@login_required
def upload():
    global data
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database
)
    uploadedBy = current_user.id
    mycursor = mydb.cursor()
    image_name = request.form['imgname']
    file = request.files['image']
    image_file = request.files['image'].stream.read()
    if not image_name == '':
        file.filename = image_name + os.path.splitext(file.filename)[1]

    if file.filename == '':
        return '<h1>Please Select a File to Upload! <meta http-equiv="refresh" content="2;url=http://euclid.nmu.edu:'+str(port)+'/form"/> </h1>'
    
    if image_file and allowed_file(file.filename):
        filename = secure_filename(file.filename)
        sql = "INSERT INTO images (image_file,user_id,image_name) VALUES(%s,%s,%s)"
        val = (image_file,uploadedBy,filename)
        mycursor.execute(sql, val)
        mydb.commit()


    else:
        return '<h1>File Type Not Supported! <meta http-equiv="refresh" content="2;url=http://euclid.nmu.edu:'+str(port)+'/form"/> </h1>'      
    return redirect(url_for('index'))

# Fetches the images from from the database and displays them
@app.route("/index")
@login_required
def index():
    mydb = connect()
    pic = ""
    mycursor = mydb.cursor()
    mycursor.execute("select image_id from images")
    pic= mycursor.fetchall()
    mycursor.close()
    mydb.close()

    return render_template("collage.html",pic=pic, name=current_user.username)

# Creats a thumbnail for each image ti be displayed
@app.route("/thumbnail/<int:id>")
@login_required
def thumbnail(id):
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database )

    mycursor = mydb.cursor()
    mycursor.execute("select image_file from images where image_id = %s",(id,));
    img= mycursor.fetchall()
    mycursor.close()
    mydb.close()
    try:
        img = img[0][0]
        
        img=Image.open(BytesIO(img))
    except IndexError:
        return render_template("no-file.html",port = port)
    except UnidentifiedImageError:
        return '<h1>Currupted Image! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/'+str(current_user.username)+'/gallary"/> </h1>'
    width, height = img.size
    if img.mode in ("RGBA", "P"):
        img = img.convert("RGB")
    if height < 500:
        img = img.resize((int(500),int(500)))
        imgBytes = io.BytesIO()
        img.save(imgBytes, format='JPEG')
        imgBytes.seek(0)
        return send_file(imgBytes, attachment_filename='Custom-Res-'+str(id)+'.JPEG')
        
    else:
        imgBytes = io.BytesIO()
        img.save(imgBytes, format='JPEG')
        imgBytes.seek(0)
        return send_file(imgBytes, attachment_filename='Custom-Res-'+str(id)+'.JPEG')

# Show an image given the image id
@app.route('/show/<int:id>')
@login_required
def show(id):
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database
    )
    mycursor = mydb.cursor()
    mycursor.execute("select image_file from images where image_id = %s",(id,));
    image_file= mycursor.fetchall()
    mycursor.close()
    mydb.close()
    
    try:
        response = make_response(image_file[0][0])
    except IndexError:
        return '<h1>Image Does Not/No Longer Exists! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/'+str(current_user.username)+'/gallary"/> </h1>'

    response.headers.set('Content-Type', 'image/jpeg')
    return response

# Given an image id downloads an image in "web_maximum" quality
@app.route("/High-Quality-Download/<int:id>")
@login_required
def highQualityDownload(id):
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database )
    mycursor = mydb.cursor()
    mycursor.execute("select image_file from images where image_id = %s",(id,));
    img= mycursor.fetchall()
    mycursor.close()
    mydb.close()

    try:
        img = img[0][0]
        img=Image.open(BytesIO(img))
    except IndexError:
        return render_template("no-file.html",port = port)
    except UnidentifiedImageError:
        edit_routes.delete(id)
        return '<h1>Currupted Image! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/'+str(current_user.username)+'/gallary"/> </h1>'

    if img.mode in ("RGBA", "P"):
        img = img.convert("RGB")
    imgBytes = io.BytesIO()
    imgBytes = io.BytesIO()
    img.save(imgBytes, format='JPEG', quality = "web_maximum")
    imgBytes.seek(0)
    return send_file(imgBytes, attachment_filename='Highquality-'+str(id)+'.JPEG',as_attachment=True)

# Given an image id downloads an image in "web_high" quality
@app.route("/Average-Quality-Download/<int:id>")
@login_required
def averageQualityDownload(id):
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database )
    mycursor = mydb.cursor()
    mycursor.execute("select image_file from images where image_id = %s",(id,));
    img= mycursor.fetchall()
    mycursor.close()
    mydb.close()
    
    try:
        img = img[0][0]
        img=Image.open(BytesIO(img))
    except IndexError:
        return render_template("no-file.html",port = port)
    except UnidentifiedImageError:
        edit_routes.delete(id)
        return '<h1>Currupted Image! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/'+str(current_user.username)+'/gallary"/> </h1>'

    if img.mode in ("RGBA", "P"):
        img = img.convert("RGB")
    imgBytes = io.BytesIO()
    img.save(imgBytes, format='JPEG', quality = "web_high")
    imgBytes.seek(0)
    return send_file(imgBytes, attachment_filename='Mediumquality-'+str(id)+'.JPEG',as_attachment=True)

# Given an image id downloads an image in "web_medium" quality
@app.route("/Low-Quality-Download/<int:id>")
@login_required
def lowQualityDownload(id):
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database )
    mycursor = mydb.cursor()
    mycursor.execute("select image_file from images where image_id = %s",(id,));
    img= mycursor.fetchall()
    mycursor.close()
    mydb.close()
    
    try:
        img = img[0][0]
        img=Image.open(BytesIO(img))
    except IndexError:
        return render_template("no-file.html",port = port)
    except UnidentifiedImageError:
        edit_routes.delete(id)
        return '<h1>Currupted Image! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/'+str(current_user.username)+'/gallary"/> </h1>'

    if img.mode in ("RGBA", "P"):
        img = img.convert("RGB")
    imgBytes = io.BytesIO()
    img.save(imgBytes, format='JPEG', quality = "web_medium")
    imgBytes.seek(0)
    return send_file(imgBytes, attachment_filename='Lowquality'+str(id)+'.JPEG',as_attachment=True)

# Given an image id downloads an image in "web_low" quality
@app.route("/Compressed-Download/<int:id>")
@login_required
def compressedDownload(id):
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database )
    mycursor = mydb.cursor()
    mycursor.execute("select image_file from images where image_id = %s",(id,));
    img= mycursor.fetchall()
    mycursor.close()
    mydb.close()
    
    try:
        img = img[0][0]
        img=Image.open(BytesIO(img))
    except IndexError:
        return render_template("no-file.html",port = port)
    except UnidentifiedImageError:
        edit_routes.delete(id)
        return '<h1>Currupted Image! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/'+str(current_user.username)+'/gallary"/> </h1>'

    if img.mode in ("RGBA", "P"):
        img = img.convert("RGB")
    imgBytes = io.BytesIO()
    img.save(imgBytes, format='JPEG', quality = "web_low")
    imgBytes.seek(0)
    return send_file(imgBytes, attachment_filename='Compressed'+str(id)+'.JPEG',as_attachment=True)

# Given an image id and height and width, allows a user to download an image in custom resolution
@app.route("/Custom-Resolution/<int:id>",methods=['POST'])
@login_required
def customResolution(id):
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database )
    mycursor = mydb.cursor()
    mycursor.execute("select image_file from images where image_id = %s",(id,));
    img= mycursor.fetchall()
    mycursor.close()
    mydb.close()

    try:
        img = img[0][0]
        img=Image.open(BytesIO(img))
    except IndexError:
        return render_template("no-file.html",port = port)
    except UnidentifiedImageError:
        edit_routes.delete(id)
        return '<h1>Currupted Image! <meta http-equiv="refresh" content="3;url=http://euclid.nmu.edu:'+str(port)+'/'+str(current_user.username)+'/gallary"/> </h1>'

    height = request.form['height']
    width = request.form['width']

    
    img = img.resize((int(width), int(height)))
    if img.mode in ("RGBA", "P"):
        img = img.convert("RGB")
    imgBytes = io.BytesIO()
    img.save(imgBytes, format='JPEG')
    imgBytes.seek(0)
    return send_file(imgBytes, attachment_filename='Custom-Res-'+str(id)+'.JPEG')

# Given an image id adds a like to the likesdislikes table
@app.route("/Like/<int:id>")
@login_required
def like(id):
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database )
    mycursor = mydb.cursor()

    user = current_user.id
    sql = "INSERT INTO likesDislikes(image, likes, user_id, dislikes) VALUES(%s,1,%s,0)"
    val = (id,user)
    try:
        mycursor.execute(sql, val)
        mydb.commit()
    except:
        sql = "UPDATE likesDislikes set dislikes =0, likes=1 where image = %s AND user_id = %s"
        val = (id,user)
        mycursor.execute(sql, val)
        mydb.commit()
   
    mycursor.close()
    mydb.close()
    
    return redirect(url_for('index'))

# Given an image id adds a dislike to the likesdislikes table
@app.route("/Dislike/<int:id>")
@login_required
def dislike(id):
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database )
    user = current_user.id
    mycursor = mydb.cursor()
    sql = "INSERT INTO likesDislikes(image, likes, user_id, dislikes) VALUES(%s,0,%s,1)"
    val = (id,user)
    try:
        mycursor.execute(sql, val)
        mydb.commit()
    except:
        sql = "UPDATE likesDislikes set dislikes =1, likes=0 where image = %s AND user_id = %s"
        val = (id,user)
        mycursor.execute(sql, val)
        mydb.commit()
    mycursor.close()
    mydb.close()
    return redirect(url_for('index'))

# Shows the dislikes an image has
@app.route('/showDislikes/<int:id>')
@login_required
def showDislikes(id):
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database
    )
    mycursor = mydb.cursor()
    mycursor.execute("select dislikes from likesDislikes where image = %s AND dislikes=1",(id,));
    likes= mycursor.rowcount
    mycursor.close()
    mydb.close()
    return 'Dislikes:'+str(likes)

# shows the likes an image has
@app.route('/showLikes/<int:id>')
@login_required
def showLikes(id):
    mydb = MySQLdb.connect(
    host=cred.host,
    user=cred.user,
    password=cred.password,
    database=cred.database
    )
    mycursor = mydb.cursor()
    mycursor.execute("select likes from likesDislikes where image = %s AND likes=1",(id,));
    likes= mycursor.rowcount
    mycursor.close()
    mydb.close()
    return 'Likes: '+str(likes)

# Shows only the images a user has uploaded
@app.route("/<userName>/gallary")
@login_required
def gallary(userName):
    mydb = MySQLdb.connect(host=cred.host,user=cred.user,password=cred.password,database=cred.database )
    mycursor = mydb.cursor()

    pic = ""
   
    mycursor.execute("select id from usernames where username = %s",(userName,))
    tempID= mycursor.fetchall()

    try:
        tempID=tempID[0][0]
    except IndexError:
        mycursor.close()
        mydb.close()
        return render_template("NoSuchUser.html",port = port)
    
    mycursor.execute("select image_id from images where user_id = %s",(tempID,))
    pic= mycursor.fetchall()
    mycursor.close()
    mydb.close()
    return render_template("userGallary.html",pic=pic,name=current_user.username)



# route that directs to th profile of the current useres profiles
@app.route('/profile')
@login_required
def profile():
    mydb = MySQLdb.connect(host=cred.host,user=cred.user,password=cred.password,database=cred.database )
    mycursor = mydb.cursor()
    mycursor.execute("select image from likesDislikes where user_id = %s AND likes = 1",(current_user.id,))

    likedImages= mycursor.fetchall()
    mycursor.close()
    mydb.close()
    return render_template('profile.html', name=current_user.username, likedImages = likedImages)


if __name__ == '__main__':
    app.run('0.0.0.0', edit_routes.port, True)